<?php
/**
 *  2011-2020 DMConcept
 *
 *  @author    DMConcept <support@dmconcept.fr>
 *  @copyright 2020 DMConcept
 *  @license   Commercial
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once _PS_MODULE_DIR_ . 'antsroute/classes/AntsRouteManager.php';


class AntsRoute extends Module
{

    protected $configurationKeys = array(
            'ANTSROUTE_CARRIER'                  => 0,
            'ANTSROUTE_DELIVERY_TRIGGER'         => '',
            'ANTSROUTE_CANCEL_TRIGGER'           => '',
            'ANTSROUTE_API_KEY'                  => '',
            'ANTSROUTE_ADDING_ORDER'             => '',
            'ANTSROUTE_DELIVERY_WEEK_TIME_SLOTS' => '{"1":{"start":"","end":""},"2":{"start":"","end":""},
            "3":{"start":"","end":""},"4":{"start":"","end":""},
            "5":{"start":"","end":""},"6":{"start":"","end":""},
            "0":{"start":"","end":""}}',
            'ANTSROUTE_DELIVERY_DEADLINE'        => 0,
            'ANTSROUTE_DELIVERY_TIME'            => 0,
            'ANTSROUTE_ENABLE_PICKUP_DELIVERY'   => 0,
            'ANTSROUTE_DEPOSIT_ADDRESS'          => '',
            'ANTSROUTE_DEPOSIT_NAME'             => '',
            'ANTSROUTE_CHARGING_TIME'            => 0,
            'ANTSROUTE_CHARGING_TIME_SLOTS'      => '{"start":"","end":""}'
        );

    protected $hooks = array(
        'actionOrderStatusPostUpdate',
        'displayAdminOrderLeft'
    );

    /**
     * AntsRoute constructor.
     */
    public function __construct()
    {
        $this->name = 'antsroute';
        $this->displayName = $this->l('AntsRoute');
        $this->description = $this->l('An essential module to simplify the organization of your deliveries.'
        . ' Build optimized routes in minutes to improve the productivity '
        . 'of your delivery people and customer satisfaction.');
        $this->tab = 'front_office_features';
        $this->version = '1.0.0';
        $this->ps_versions_compliancy = array(
            'min' => '1.6.0.4',
            'max' => _PS_VERSION_
        );
        $this->author = 'DMConcept';
        $this->need_instance = 1;
		$this->module_key = 'd9afd5d382d4c6443035fb68acad1564';
        $this->bootstrap = true;

        parent::__construct();

        /*
         * DOCUMENTATION : https://app.antsroute.com/doc-api/index.html
         */
    }

    public function hookDisplayAdminOrderLeft($params)
    {
        if (Tools::getValue('action') === 'sendOrder') {
            ob_clean();
            $this->ajaxProcessSendOrder();
            die();
        }

        $order = new Order((int)$params['id_order']);

        try {
            $order->delivery_date = Tools::displayDate($order->delivery_date);
        } catch (PrestaShopException $e) {
        }

        if (Validate::isLoadedObject($order)
            && ($carrier = new Carrier((int)$order->id_carrier))
            && Validate::isLoadedObject($carrier)
            && $carrier->id_reference == Configuration::get('ANTSROUTE_CARRIER')
        ) {
            $antsRouteOrderState = AntsRouteManager::getInstance()->getAntsRouteOrderState($params['id_order']);

            $this->context->controller->addCSS(_PS_MODULE_DIR_
                . 'antsroute/views/css/hook/display-admin-order-left.css');
            $this->context->controller->addJS(_PS_MODULE_DIR_ . 'antsroute/views/js/hook/display-admin-order-left.js');

            return $this->context->smarty->fetch(
                _PS_MODULE_DIR_ . 'antsroute/views/templates/hook/display-admin-order-left.tpl',
                array(
                    'antsRouteOrderState' => $antsRouteOrderState,
                    'id_order' => $params['id_order'],
                    'deliveryDate' => $order->delivery_date
                )
            );
        }
    }

    public function ajaxProcessSendOrder()
    {
        $added = AntsRouteManager::getInstance()->triggerDelivery(Tools::getValue('id_order'));
        if ($added && is_null($added['data']['errorCode'])) {
            $deliveryDate = empty($added['data']['dueDate'])
                ? $added['data']['scheduledDate'] : $added['data']['dueDate'];
            try {
                $deliveryDate = Tools::displayDate($deliveryDate);
            } catch (PrestaShopException $e) {
            }
            die(json_encode(array_merge(array('success' => true, 'deliveryDate' => $deliveryDate), $added['data'])));
        }
        die(json_encode(array('error' => true)));
    }

    /**
     * @param $params
     *
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function hookActionOrderStatusPostUpdate($params)
    {
        $newStatus = $params['newOrderStatus'];
        $idNewOrderStatus = $newStatus->id;
        $idOrder = $params['id_order'];

        if ((int)$idNewOrderStatus === (int)Configuration::get('ANTSROUTE_DELIVERY_TRIGGER')) {
            AntsRouteManager::getInstance()->triggerDelivery($idOrder);
        } elseif ((int)$idNewOrderStatus === (int)Configuration::get('ANTSROUTE_CANCEL_TRIGGER')) {
            AntsRouteManager::getInstance()->triggerCancel($idOrder);
        }
    }


    /**
     * @return mixed|null
     */
    public static function getVersionMajor()
    {
        static $version = null;

        if ($version == null) {
            $version_with_point = Tools::substr(_PS_VERSION_, 0, 3);
            $version = str_replace('.', '', $version_with_point);
        }

        return $version;
    }


    /**
     * Load the configuration form
     */
    public function getContent()
    {
        $redirect_after = $this->context->link->getAdminLink('AdminAntsRouteConfig');
        Tools::redirectAdmin($redirect_after);
    }


    /**
     *
     * @return bool
     */
    public function install()
    {
        include(dirname(__FILE__).'/sql/install.php');


        $parent = parent::install();

        $successConfig = true;
        foreach ($this->configurationKeys as $key => $value) {
            $successConfig &= Configuration::updateValue($key, $value);
        }

        $successHook = true;
        foreach ($this->hooks as $hook) {
            $successHook &= $this->registerHook($hook);
        }


        return $parent && $successHook && $successConfig && $this->installTabs();
    }

    /**
     * @return bool
     */
    public function installTabs()
    {
        $success = $this->installModuleTab(
            'AdminAntsRoute',
            $this->l('AntsRoute'),
            1,
            (version_compare(_PS_VERSION_, '1.7.0.0', '>=') ? 'IMPROVE' : 0),
            'local_shipping'
        );
        $success &= $this->installModuleTab(
            'AdminAntsRouteConfig',
            $this->l('Setting'),
            1,
            'AdminAntsRoute',
            'local_shipping'
        );
        $success &= $this->installModuleTab(
            'AdminAntsRouteLogs',
            $this->l('Logs'),
            1,
            'AdminAntsRoute',
            'local_shipping'
        );
        return $success;
    }

    /**
     *
     * @return bool
     */
    public function uninstall()
    {
        include(dirname(__FILE__).'/sql/uninstall.php');

        foreach ($this->configurationKeys as $key) {
            if (!Configuration::deleteByName($key)) {
                //return false;
            }
        }

        return parent::uninstall();
    }

    /**
     * @return bool
     */
    public function reset()
    {
        if ($this->uninstall(true)) {
            return $this->install(true);
        }
        return false;
    }

    /**
     * @param  bool  $force_all
     *
     * @return bool
     */
    public function disable($force_all = false)
    {
        if (self::getVersionMajor() < 17) {
            $this->uninstallOverrides();
        }
        return parent::disable($force_all);
    }

    /**
     * @param  bool  $force_all
     *
     * @return bool
     */
    public function enable($force_all = false)
    {
        if (self::getVersionMajor() < 17) {
            $this->uninstallOverrides();
            $this->installOverrides();
        }
        return parent::enable($force_all);
    }

    /**
     * @param          $tabClass
     * @param          $tabName
     * @param  int     $active
     * @param  int     $tabParent
     * @param  string  $tabIcon
     *
     * @return bool
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function installModuleTab(
        $tabClass,
        $tabName,
        $active = 1,
        $tabParent = 0,
        $tabIcon = ''
    ) {
        $this->uninstallModuleTab($tabClass);

        $idTabParent = $tabParent ? Tab::getIdFromClassName($tabParent) : 0;
        $tab = new Tab();
        $languages = Language::getLanguages();
        foreach ($languages as $language) {
            $tab->name[(int)$language['id_lang']] = $tabName;
        }

        $tab->class_name = $tabClass;
        $tab->module = $this->name;
        $tab->id_parent = $idTabParent;
        $tab->active = $active;
        $tab->icon = $tabIcon;
        if (!$tab->save()) {
            return false;
        }
        return true;
    }

    /**
     * @param $tabClass
     *
     * @return bool
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function uninstallModuleTab($tabClass)
    {
        $idTab = Tab::getIdFromClassName($tabClass);
        if ($idTab != 0) {
            $tab = new Tab($idTab);
            $tab->delete();
            return true;
        }
        return false;
    }


    /**
     * @param $table_name
     * @param $column_name
     *
     * @return bool
     * @throws PrestaShopDatabaseException
     */
    public function existColumnInTable($table_name, $column_name)
    {
        $sql = 'DESCRIBE '._DB_PREFIX_.$table_name;
        $columns = Db::getInstance()->executeS($sql);
        $found = false;

        foreach ($columns as $col) {
            if ($col['Field'] == $column_name) {
                $found = true;
                break;
            }
        }

        return $found;
    }

    /**
     * @param $table_name
     *
     * @return bool
     * @throws PrestaShopDatabaseException
     */
    public function existTableInDatabase($table_name)
    {
        $sql = 'SHOW TABLES';
        $tables = Db::getInstance()->executeS($sql);

        foreach ($tables as $table) {
            $table_values = array_values($table);
            $table_find = isset($table_values[0]) ? $table_values[0] : '';

            if ($table_find == _DB_PREFIX_.$table_name) {
                return true;
            }
        }

        return false;
    }

    /**
     * @param $table_name
     * @param $column_name
     *
     * @return bool
     * @throws PrestaShopDatabaseException
     */
    public function existIndexInTable($table_name, $column_name)
    {
        $sql = 'SHOW INDEX FROM `'._DB_PREFIX_.$table_name.'`';
        $indexes = Db::getInstance()->executeS($sql);

        foreach ($indexes as $index) {
            if (isset($index['Column_name'])
                && $index['Column_name'] == $column_name
            ) {
                return true;
            }
        }

        return false;
    }
}
