<?php
/**
 *  2011-2020 DMConcept
 *
 *  @author    DMConcept <support@dmconcept.fr>
 *  @copyright 2020 DMConcept
 *  @license   Commercial
 */

if (!defined('_CAN_LOAD_FILES_')) {
    exit;
}

require_once _PS_MODULE_DIR_ . 'antsroute/classes/AntsRouteApiConnection.php';

if (!class_exists('AntsRouteManager')) {
    class AntsRouteManager
    {
        private static $instance;

        const INTERVENTION_BASKET = 1;
        const MODULE_ORGANIZE = 2;

        private function __construct()
        {
        }

        public function getExternalId($idOrder)
        {
            return Tools::substr((string)Configuration::get('ANTSROUTE_API_KEY'), 0, 10) . '-' . $idOrder;
        }

        /**
         * @return AntsRouteManager
         */
        public static function getInstance()
        {
            if (!isset(self::$instance)) {
                self::$instance = new AntsRouteManager();
            }
            return self::$instance;
        }

        /**
         * @return bool|false|string
         */
        protected function selectDueDate()
        {
            $numberOfDayForDelivery = (int)Configuration::get('ANTSROUTE_DELIVERY_DEADLINE');
            $deliveryWeekTimeSlots = json_decode(Configuration::get('ANTSROUTE_DELIVERY_WEEK_TIME_SLOTS'), true);

            $hasValidDate = false;
            foreach ($deliveryWeekTimeSlots as $timeSlot) {
                if (!empty($timeSlot['start']) && !empty($timeSlot['end'])) {
                    $hasValidDate = true;
                }
            }
            if (!$hasValidDate) {
                return false;
            }

            do {
                $dueDate = date('Y-m-d', strtotime("+$numberOfDayForDelivery days"));
                $dueDateDayNumber = date('w', strtotime($dueDate));

                $timeSlot = $deliveryWeekTimeSlots[$dueDateDayNumber];

                if (!empty($timeSlot['start']) && !empty($timeSlot['end'])) {
                    return $dueDate;
                }

                $numberOfDayForDelivery++;
            } while (true);
            // i am sure that the will end with a result because list is tested before while loop
        }

        public function updateDelivery($order, $date)
        {
            $order->setDelivery();
            $order_invoice_collection = $order->getInvoicesCollection();
            foreach ($order_invoice_collection as $order_invoice) {
                $order_invoice->delivery_date = $date;
                $order_invoice->save();
            }
            $order->delivery_date = $date;
            $order->save();
        }

        /**
         * @param $idOrder
         *
         * @return bool|mixed
         * @throws PrestaShopDatabaseException
         * @throws PrestaShopException
         */
        public function triggerDelivery($idOrder)
        {
            // get config variables
            $deliveryWeekTimeSlots = json_decode(Configuration::get('ANTSROUTE_DELIVERY_WEEK_TIME_SLOTS'), true);
            $duration = (int)Configuration::get('ANTSROUTE_DELIVERY_TIME');

            // get prestashop object
            $antsRouteCarrier = Carrier::getCarrierByReference((int)Configuration::get('ANTSROUTE_CARRIER'));
            $psOrder = new Order((int)$idOrder);
            if (Validate::isLoadedObject($psOrder)) {
                $psCustomer = new Customer((int)$psOrder->id_customer);
                $psCustomerAddress = new Address((int)$psOrder->id_address_delivery);
            }

            if (!$antsRouteCarrier || (int)$psOrder->id_carrier !== (int)$antsRouteCarrier->id
                || !Validate::isLoadedObject($psOrder)
                || !Validate::isLoadedObject($psCustomer)
                || !Validate::isLoadedObject($psCustomerAddress)
            ) {
                return false;
            }


            // set request data
            $dueDate = $this->selectDueDate();
            $dueDateDayNumber = date('w', strtotime($dueDate));
            $address = $psCustomerAddress->address1 . ' ' .  $psCustomerAddress->address2
                . ' ' . $psCustomerAddress->postcode . ' ' . $psCustomerAddress->city;
            $customer = array(
                'lastName'   => $psCustomer->lastname,
                'firstName'  => $psCustomer->firstname,
                'address'    => $address,
                'externalId' => $psCustomer->id,
                'email'      => $psCustomer->email
            );
            $type = (int)Configuration::get("ANTSROUTE_ENABLE_PICKUP_DELIVERY") ? 'DELIVERY' : 'SERVICE';
            $comments = $psOrder->getFirstMessage();

            $weight = 0;
            $numberOfProduct = 0;
            $description = '';
            foreach ($psOrder->getProducts() as $product) {
                $description .= $product['reference'] . " ";
                $weight += $product['weight'];
                $numberOfProduct += $product['product_quantity'];
            }
            $customFields = array(
                array('name' => 'Description', 'value' => empty($description)
                    ? $description : str_replace(
                        " ",
                        " - ",
                        Tools::substr($description, 0, Tools::strlen($description) - 1)
                    )
                ),
                array('name' => 'Price', 'value' => $psOrder->total_paid),
                array('name' => 'Reference', 'value' => 19)
            );

            $capacities = array(
                array('capacityName' => 'Number', 'capacityValue' => $numberOfProduct),
                array('capacityName' => 'Weight', 'capacityValue' => $weight),
            );


            $url = (int)Configuration::get('ANTSROUTE_ADDING_ORDER') === self::INTERVENTION_BASKET
                ? 'capi/order/basket' : 'capi/order/planning';

            $data = array(
                'dueDate'      => $dueDate,
                'scheduleDate' => $dueDate,
                'type'         => $type,
                'duration'     => $duration,
                'externalId'   => $this->getExternalId($idOrder),
                'timeSlot'     => $deliveryWeekTimeSlots[$dueDateDayNumber],
                'customer'     => $customer,
                'comments'     => $comments ? $comments : "",
                'customFields' => $customFields,
                'capacities'   => $capacities,
            );
            if ((int)Configuration::get("ANTSROUTE_ENABLE_PICKUP_DELIVERY")) {
                $data['loading'] = array(
                    'location' => array(
                        'name'    => Configuration::get('ANTSROUTE_DEPOSIT_NAME'),
                        'address' => Configuration::get('ANTSROUTE_DEPOSIT_ADDRESS')
                    ),
                    'duration' => (int)Configuration::get('ANTSROUTE_CHARGING_TIME'),
                    'timeSlot' => json_decode(Configuration::get('ANTSROUTE_CHARGING_TIME_SLOTS'), true)
                );
            }

            if (!$dueDate) {
                return false;
            }

            $this->updateDelivery($psOrder, $dueDate);

            return AntsRouteApiConnection::getInstance()->send($url, 'POST', $data, 'application/json');
        }

        /**
         * @param $idOrder
         *
         * @return bool|mixed
         */
        public function triggerCancel($idOrder)
        {
            return AntsRouteApiConnection::getInstance()->send('capi/order/external-id/' .
                $this->getExternalId($idOrder) . '?purge=true', 'DELETE');
        }


        /**
         * @param $idOrder
         *
         * @return bool|string
         */
        public function getAntsRouteOrderState($idOrder)
        {
            $antsRouteOrder =  AntsRouteApiConnection::getInstance()->send('capi/order/external-id/'
                . $this->getExternalId($idOrder), 'GET');

            if ($antsRouteOrder && Tools::substr($antsRouteOrder['status'], 0, 1) == 2) {
                return trim($antsRouteOrder['data']['state']);
            }
            return false;
        }
    }
}
